 /* **********************************************************************************
 * Project Name: Verification Environment for Robot Controller.
 * File Name:    rc_agent.svh
 * Description:  Class represents Agent for Robot Controller.
 * Author:       Stanislav Krajcir <xkrajc06@stud.fit.vutbr.cz>
 * Supervisor:	 Marcela Simkova <isimkova@fit.vutbr.cz>   
 * Date:         25.5.2015
 * **********************************************************************************/


 
 class rcAgent extends uvm_component;
    
   //! UVM Factory Registration Macro
   `uvm_component_utils(rcAgent)
   
  /*! 
   * Data Members
   */  
   
   rcAgentConfig  rc_agent_cfg;
   TransactionSequenceConfig transaction_sequence_cfg; 
  
   
  /*! 
   * Ports/Exports
   */ 
   
   uvm_analysis_port #(rcInputTransaction)  ap_in;  
   uvm_analysis_port #(rcOutputTransaction) ap_out;  
   
  /*!
   * Component Members
   */  
   
  TransactionSequencer  trans_sequencer;
  rcDriver              rc_driver; 
  rcMonitor				rc_monitor;
   
  /*!
   * Methods
   */
   
   // Standard UVM methods
   extern function new(string name = "rcAgent", uvm_component parent = null);
   extern function void build_phase(uvm_phase phase);
   extern function void connect_phase(uvm_phase phase);
   extern function void end_of_elaboration_phase(uvm_phase phase);
      
   // Own UVM methods
  extern function void configure_transaction_sequence(TransactionSequenceConfig transaction_sequence_cfg);
   
 endclass: rcAgent



/*! 
 * Constructor - creates the rcAgent object  
 */
 function rcAgent::new(string name = "rcAgent", uvm_component parent = null);
   super.new(name, parent);
 endfunction: new



/*! 
 * Build - environment configuration
 */ 
 function void rcAgent::build_phase(uvm_phase phase);
   // check configuration for the Agent
   if (!uvm_config_db #(rcAgentConfig)::get(this, "", "rcAgentConfig", rc_agent_cfg)) 
     `uvm_error("MYERR", "rcAgentConfig doesn't exist!"); 
     
   // create configuration object for Transaction Sequence
   transaction_sequence_cfg = TransactionSequenceConfig::type_id::create("transaction_sequence_cfg");
   
   
   // configure transaction sequence
   configure_transaction_sequence(transaction_sequence_cfg);
   
   
   // put configuration into the configuration space
   uvm_config_db #(TransactionSequenceConfig)::set(this, "*", "TransactionSequenceConfig", transaction_sequence_cfg);  
    
   
   // create analysis ports
   ap_in  = new("ap_in",this);
   ap_out = new("ap_out",this);   
     
   // >>>>> CREATE COMPONENTS >>>>>
   trans_sequencer = TransactionSequencer::type_id::create("TransactionSequencer", this);
   rc_driver = rcDriver::type_id::create("rcDriver", this); 
   rc_monitor = rcMonitor::type_id::create("rcMonitor", this);
 endfunction: build_phase



/*! 
 * Connect - interconnection of Agent components.
 */  
 function void rcAgent::connect_phase(uvm_phase phase);
   // TLM connection
   rc_driver.seq_item_port.connect(trans_sequencer.seq_item_export);
   
   // analysis port connections
   ap_in  = rc_driver.aport_rc_in_if;
   ap_out = rc_monitor.aport_rc_out_if;
 endfunction: connect_phase
 

 
/*! 
 * end_of_elaboration_phase
 */  
 function void rcAgent::end_of_elaboration_phase(uvm_phase phase);
 endfunction: end_of_elaboration_phase
 
 

/*! 
 * Configure Transaction Sequence.
 */   
 function void rcAgent::configure_transaction_sequence(TransactionSequenceConfig transaction_sequence_cfg);
   transaction_sequence_cfg.trans_count     = rc_agent_cfg.trans_count;
 endfunction: configure_transaction_sequence
 
 
