 /* **********************************************************************************
 * Project Name: Verification Environment for Robot Controller.
 * File Name:    rc_driver.svh
 * Description:  Class represents Driver for Robot Controller.
 * Author:       Stanislav Krajcir <xkrajc06@stud.fit.vutbr.cz>
 * Supervisor:	 Marcela Simkova <isimkova@fit.vutbr.cz>   
 * Date:         25.5.2015
 * **********************************************************************************/
/*
 * Robot Controller Driver
 * 
 * This class drives the input interface of RC and supplies transactions from 
 * sequencer to this interface.
 */

 class rcDriver extends uvm_driver #(rcInputTransaction);

   //! UVM Factory Registration Macro
   `uvm_component_utils(rcDriver)
  
  /*! 
   * Virtual interfaces for DUT
   */ 
   
   virtual interfaceRCin dut_rc_in_if;
  
  /*! 
   * Ports/Exports
   */ 
   
   uvm_analysis_port #(rcInputTransaction) aport_rc_in_if;
   
  /*!
   * Data Members
   */ 
   
   rcAgentConfig rc_agent_cfg;
      
  /*!
   * Methods
   */
   
   // Standard UVM methods
   extern function new(string name = "rcDriver", uvm_component parent = null);
   extern function void build_phase(uvm_phase phase);
   extern function void connect_phase(uvm_phase phase); 
   extern task run_phase(uvm_phase phase); 
   
   // Own UVM methods

   
 endclass: rcDriver



/*! 
 * Constructor - creates the rcDriver object  
 */
 function rcDriver::new(string name = "rcDriver", uvm_component parent = null);
   super.new(name, parent);
 endfunction: new
   
 

/*! 
 * Build - instanciates child components
 */ 
 function void rcDriver::build_phase(uvm_phase phase);
   if (!uvm_config_db #(rcAgentConfig)::get(this, "", "rcAgentConfig", rc_agent_cfg)) 
     `uvm_error("MYERR", "rcAgentConfig doesn't exist!");
     
   // create analysis port
    aport_rc_in_if = new("aport_rc_in_if", this); 
 endfunction: build_phase   



/*! 
 * Connect - interconnection of rcDriver and DUT.
 */  
 function void rcDriver::connect_phase(uvm_phase phase);
   super.connect_phase(phase);
   dut_rc_in_if = rc_agent_cfg.dut_rc_in_if;
 endfunction: connect_phase
 
 
 
/*! 
 * Run - starts driver processing.
 */  
 task rcDriver::run_phase(uvm_phase phase); 
   rcInputTransaction rc_in_trans;
   
   // synchronise with CLK 
   @(dut_rc_in_if.cb); 
   
   forever begin
     seq_item_port.get_next_item(rc_in_trans);
     
            
     rc_in_trans.rst = dut_rc_in_if.RST;
     
     // display the content of transaction 
	  rc_in_trans.print("DRIVER: INPUT TRANSACTION");
       
     // set input signals of DUT
     // sends values from transaction on the virtual interface
		 	 
	 dut_rc_in_if.cb.DIST_A <= rc_in_trans.dist_a;
	 dut_rc_in_if.cb.DIST_B <= rc_in_trans.dist_b;
	 dut_rc_in_if.cb.DIST_C <= rc_in_trans.dist_c;
	 dut_rc_in_if.cb.S_0	<= rc_in_trans.s_0;
	 dut_rc_in_if.cb.S_1	<= rc_in_trans.s_1;
	 dut_rc_in_if.cb.S_2	<= rc_in_trans.s_2;
	 dut_rc_in_if.cb.S_3	<= rc_in_trans.s_3;
	 dut_rc_in_if.cb.DATA_VLD <= rc_in_trans.data_vld;
								 
	 

     
     // sends generated transaction to the scoreboard.
     if (rc_in_trans.data_vld) aport_rc_in_if.write(rc_in_trans);
              
     // synchronise with CLK 
     @(dut_rc_in_if.cb); 
     
     seq_item_port.item_done();
   end   
 endtask: run_phase 