 /* **********************************************************************************
 * Project Name: Verification Environment for Robot Controller.
 * File Name:    rc_monitor.svh
 * Description:  Class represents Monitor for Robot Controller.
 * Author:       Stanislav Krajcir <xkrajc06@stud.fit.vutbr.cz>
 * Supervisor:	 Marcela Simkova <isimkova@fit.vutbr.cz>   
 * Date:         25.5.2015
 * **********************************************************************************/

 /*
 * rcMonitor
 * 
 * This class monitors signals on RC output interface and sends their values 
 * in the form of transactions to the Scoreboard.
 */

 class rcMonitor extends uvm_monitor;

   //! UVM Factory Registration Macro
   `uvm_component_utils(rcMonitor)

  /*! 
   * Virtual interfaces for DUT
   */ 
  
   virtual interfaceRCout dut_rc_out_if;
   
  /*! 
   * Ports/Exports
   */ 
   
   uvm_analysis_port #(rcOutputTransaction) aport_rc_out_if;
  
  /*! 
   * Data Members
   */
  
   rcAgentConfig rc_agent_cfg; // agent configuration
   
  /*!
   * Methods
   */
   
   // Standard UVM methods
   extern function new(string name = "rcMonitor", uvm_component parent = null);
   extern function void build_phase(uvm_phase phase);
   extern function void connect_phase(uvm_phase phase); 
   extern task run_phase(uvm_phase phase); 
   
   // OWN UVM methods
   //extern task waitForVld();
   
 endclass: rcMonitor
 
 
 
/*! 
 * Constructor - creates the rcMonitor object  
 */
 function rcMonitor::new(string name = "rcMonitor", uvm_component parent = null);
   super.new(name, parent);
 endfunction: new



/*! 
 * Build - instanciates child components
 */ 
 function void rcMonitor::build_phase(uvm_phase phase);
   super.build_phase(phase);
   
   // get configuration
   if (!uvm_config_db #(rcAgentConfig)::get(this, "", "rcAgentConfig", rc_agent_cfg)) 
     `uvm_error("MYERR", "rcAgentConfig doesn't exist!");
     
   // create analysis port
   aport_rc_out_if = new("aport_rc_out_if", this);  
 endfunction: build_phase   
 
 
 
/*! 
 * Connect - interconnection of rcMonitor components.
 */  
 function void rcMonitor::connect_phase(uvm_phase phase);
   super.connect_phase(phase);
   dut_rc_out_if = rc_agent_cfg.dut_rc_out_if;
 endfunction: connect_phase
 
 

/*! 
 * Run - starts monitor processing.
 */  
 task rcMonitor::run_phase(uvm_phase phase); 
   rcOutputTransaction rc_out_trans, rc_out_trans_c;    
   
   // check RESET
   while (dut_rc_out_if.RST) 
     @(dut_rc_out_if.cb); 
   
   // create output transaction  
   rc_out_trans = rcOutputTransaction::type_id::create();  
   
   forever
     begin
       // clone output transaction
       assert($cast(rc_out_trans_c, rc_out_trans.clone));
	   
       // receive the value of output
       rc_out_trans_c.x_speed = dut_rc_out_if.cb.X_SPEED;
       rc_out_trans_c.y_speed = dut_rc_out_if.cb.Y_SPEED;
	   rc_out_trans_c.movement = dut_rc_out_if.cb.MOVEMENT;
	   
	   // send the output transaction to the Player/Stage through DPI
	   //send_data(rc_out_trans_c.x_speed,rc_out_trans_c.y_speed);
	   

	   if (0 != send_data(rc_out_trans_c.x_speed,rc_out_trans_c.y_speed))
		begin
				// FATAL ERROR
				uvm_report_fatal("MONITOR","Failed on sending data: DPI from Player/Stage \n");
		end

	   
	   
	   if (rc_out_trans_c.movement == 1)
	   begin
		
			// display the content of transaction 
			//rc_out_trans_c.print("MONITOR: OUTPUT TRANSACTION"); 
       
			// sends generated transaction to the scoreboard
			aport_rc_out_if.write(rc_out_trans_c);
		
	   end
	   
	   
	   // ERROR RC should not move while movement is set to 0
	   if (rc_out_trans_c.movement == 0)
	   begin
			if((rc_out_trans_c.x_speed != 0) &&( rc_out_trans_c.y_speed != 0))
			begin
				uvm_report_error("MONITOR","Robot movement not allowed while signal MOVEMENT is set to \"0\"\n");
			end
	   end
	   
	   
       
       // synchronisation with the DUT
       @(dut_rc_out_if.cb);
     end
 endtask: run_phase 
 