 /* **********************************************************************************
 * Project Name: Verification Environment for Robot Controller.
 * File Name:    rc_scoreboard.svh
 * Description:  Class represents Scoreboard for Robot Controller.
 * Author:       Stanislav Krajcir <xkrajc06@stud.fit.vutbr.cz>
 * Supervisor:	 Marcela Simkova <isimkova@fit.vutbr.cz>   
 * Date:         25.5.2015
 * **********************************************************************************/

/*
 *  rcScoreboard
 * 
 * This class receives transactions from Driver and sends them to the Golden Model.
 * Afterwards, it receives transactions from Monitor and
 * automatically compares these real responses with the expected ones. 
 */

 class rcScoreboard extends uvm_scoreboard;

   //! UVM Factory Registration Macro
   `uvm_component_utils(rcScoreboard)
   
  /*! 
   * Ports/Exports
   */ 
   
   uvm_analysis_export #(rcInputTransaction)  export_rc_in_if;
   uvm_analysis_export #(rcOutputTransaction) export_rc_out_if;
   
   uvm_tlm_analysis_fifo #(rcInputTransaction)  driver_fifo;
   uvm_tlm_analysis_fifo #(rcOutputTransaction) monitor_fifo;
   
  /*!
   * Data Members
   */  
   
   // enum for comparision of received values
   enum { X_COORDINATE = 33, Y_COORDINATE = 66 } type_of_required_value;
   enum { GOAL_REACHED = 123 } goal;
   
   int i;
   int m_matches, m_mismatches;
   int goal_flag=0;
   
  /*!
   * Methods
   */
   
   // Standard UVM methods
   extern function new(string name = "rcScoreboard", uvm_component parent = null);
   extern function void build_phase(uvm_phase phase);
   extern function void connect_phase(uvm_phase phase); 
   extern task run_phase(uvm_phase phase); 
   extern function void report_phase(uvm_phase phase);
   
 endclass: rcScoreboard   
   
/*! 
 * Constructor - creates the rcScoreboard object  
 */
 function rcScoreboard::new(string name = "rcScoreboard", uvm_component parent = null);
   super.new(name, parent);
   m_matches    = 0;
   m_mismatches = 0;
 endfunction: new
 
 
    
/*! 
 * Build - instanciates child components
 */ 
 function void rcScoreboard::build_phase(uvm_phase phase);
   export_rc_in_if  = new("export_rc_in_if",this);
   export_rc_out_if = new("export_rc_out_if",this);
     
   driver_fifo  = new("driver_fifo",this);
   monitor_fifo = new("monitor_fifo",this);
 endfunction: build_phase   
   


/*! 
 * Connect - interconnection of rcScoreboard components.
 */  
 function void rcScoreboard::connect_phase(uvm_phase phase);
   export_rc_in_if.connect(driver_fifo.analysis_export);
   export_rc_out_if.connect(monitor_fifo.analysis_export);
 endfunction: connect_phase
 
 
  
/*! 
 * Run - starts scoreboard processing.
 */  
 task rcScoreboard::run_phase(uvm_phase phase); 
   string msg;
   rcInputTransaction      rc_in_tr;
   rcOutputTransaction     rc_out_tr_exp, rc_out_tr_exp_c;  // expected transaction 
   rcOutputTransaction     rc_out_tr_real;                   // real transaction
     
   // create expected output transaction
   rc_out_tr_exp = rcOutputTransaction::type_id::create();
   
	// set goal for Golden Model
	//$write("setting goal:X=%d, Y=%d\n",GOAL_X,GOAL_Y);
	set_goal(GOAL_X,GOAL_Y);
   
   forever begin
    
	// clear variable
	i=0; 
	
	
     // receive input transaction from Driver
     driver_fifo.get(rc_in_tr);
      
     // display received transaction
    // rc_in_tr.print("SCOREBOARD: INPUT TRANSACTION FROM DRIVER:");
       

     // Actual COMPARISON happens below
	 
     // clone expected output transaction
     assert($cast(rc_out_tr_exp_c, rc_out_tr_exp.clone));
	 
		rc_out_tr_exp_c.x_speed = get_speed(rc_in_tr.dist_a, 
											rc_in_tr.dist_b, 
											rc_in_tr.dist_c, 
											rc_in_tr.s_0, 
											rc_in_tr.s_1, 
											rc_in_tr.s_2, 
											rc_in_tr.s_3, 
											X_COORDINATE);
											
		rc_out_tr_exp_c.y_speed = get_speed(rc_in_tr.dist_a,
											rc_in_tr.dist_b,
											rc_in_tr.dist_c,
											rc_in_tr.s_0,
											rc_in_tr.s_1,
											rc_in_tr.s_2,
											rc_in_tr.s_3,
											Y_COORDINATE
											);															
		
		
		// display transaction received from golden model		
        rc_out_tr_exp_c.print("SCOREBOARD: VALUES RECEIVED FROM GOLDEN MODEL:");        
           

		// check for  GOAL REACHED
		if((rc_out_tr_exp_c.x_speed == GOAL_REACHED) && (rc_out_tr_exp_c.y_speed == GOAL_REACHED))
		begin
			goal_flag = GOAL_REACHED;
			$write("\n\n");
			$write("!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! \n");
			$write("!!!!!!!!!!!!!!!!!!        GOAL REACHED         !!!!!!!!!!!!!!!!!!!!! \n");
			$write("!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! \n");
			break;
		end
		   
                   // receive output trasanction from Monitor
				   while (i < TRANSACTION_COMPARE_COUT)
				   begin
				   
						monitor_fifo.get(rc_out_tr_real);
						
						if (i == TRANSACTION_COMPARE_COUT-1)
						begin
							// display expected transaction
							//rc_out_tr_exp_c.print(":SCOREBOARD EXPECTED OUTPUT:");
      
							// display received transaction
							//rc_out_tr_real.print(":SCOREBOARD   REAL  OUTPUT:");
						end
      
						// display expected transaction
						//rc_out_tr_exp_c.print(":SCOREBOARD EXPECTED OUTPUT:");
      
						// display received transaction
						//rc_out_tr_real.print(":SCOREBOARD   REAL   OUTPUT:");
       
						
	   
						// compare expected and real output transaction
						if (!rc_out_tr_real.compare(rc_out_tr_exp_c)) begin
							$sformat(msg, "Expected and real output transaction do not match!!!\n");
							uvm_report_error("SCOREBOARD", msg);
							m_mismatches++;
						end //end of if
						else m_matches++;
					
					i++;

					
					end // end of while
					
					
										
                                        
   end  // of forever
 endtask: run_phase 
 
 
 
/*! 
 * Post-body - implements closing of output file with transactions
 */ 
 function void rcScoreboard::report_phase(uvm_phase phase);
   $write("\n\n");
   $write("-------------------------------------------------------------------- \n");
   $write("------------------          SCOREBOARD         --------------------- \n");
   $write("-------------------------------------------------------------------- \n");
   
   if (m_mismatches == 0) begin
     $write("\n MATCHES: %d\n\n MISMATCHES: %d\n", m_matches, m_mismatches);
	 if (goal_flag == GOAL_REACHED)
		begin
		$write("\n VERIFICATION ENDED CORRECTLY, ROBOT REACHED GOAL COORDINATES\n\n");
		end
	 else
	 	 begin
		$write("\n VERIFICATION FAILED, ROBOT DIDN'T REACH GOAL COORDINATES\n\n");
		end	 
	 $stop();
   end
   else begin
     $write("\n MATCHES: %d\n\n MISMATCHES: %d\n", m_matches, m_mismatches);
     $write("\n SCOREBOARD MISMATCHES OCCURED IN VERIFICATION. \n\n");
	 if (goal_flag == GOAL_REACHED)
	 begin
		$write("\n VERIFICATION FAILED ALTHOUGH ROBOT REACHED GOAL COORDINATES\n\n");
	 end
	 else
	 begin
	 $write("\n VERIFICATION FAILED, ROBOT DIDN'T REACH GOAL COORDINATES\n\n");
	 end
   end
   
   $stop();
 endfunction: report_phase